class SBWarShell expands WarShell config(SBMutatorFastWarshell);

var config float ProjectileSpeed;
var config float ProjectileMaxSpeed;
var config string LaunchSoundName;
var config string UnknownDestroyerName;
var config string DestroyMessage;
var config string LaunchMessage;
var config byte ProjectileHealth;
var config bool bDebug;
var config bool bApplyFullDamage; // Apply full damage to a hit actor?
var config bool bDestroyedBroadcastMsg;
var config bool bDestroyedBroadcastMsgCritical;
var config name ValidWeaponInstigators[8];

var string InstigatorHumanName; // for later usage, if projectile is destroyed
var bool bIsValidWeaponInstigator;




replication {
	// Things the server should send to the client.
	reliable if (Role == ROLE_Authority)
		ProjectileSpeed;
}



function bool IsValidWeaponInstigator(Actor A) {
/******************************************************************************
Returns TRUE if *A* is a candidate for mutator handling.
*A* may not be NONE.
******************************************************************************/
local byte i;
local bool result;

	result = false;
	for (i = 0; i < ArrayCount(ValidWeaponInstigators); i++)
		if (ValidWeaponInstigators[i] != '')
			if (A.IsA(ValidWeaponInstigators[i]))
			{
				result = true;
				break;
			}
	if (bDebug)
		class'SBMutatorFastWarShell'.static.DebugLog(self, "IsValidWeaponInstigator returns" @ result @ "for" @ A);
	return result;
}



function LogINIValues() {
local byte i;

	class'SBMutatorFastWarShell'.static.DebugLog(self, "== INI values ==");
	class'SBMutatorFastWarShell'.static.DebugLog(self, "bDebug=" $ bDebug);
	class'SBMutatorFastWarShell'.static.DebugLog(self, "ProjectileSpeed=" $ ProjectileSpeed);
	class'SBMutatorFastWarShell'.static.DebugLog(self, "ProjectileMaxSpeed=" $ ProjectileMaxSpeed);
	class'SBMutatorFastWarShell'.static.DebugLog(self, "bApplyFullDamage=" $ bApplyFullDamage);
	class'SBMutatorFastWarShell'.static.DebugLog(self, "LaunchSoundName=" $ LaunchSoundName);
	class'SBMutatorFastWarShell'.static.DebugLog(self, "ProjectileHealth=" $ ProjectileHealth);
	class'SBMutatorFastWarShell'.static.DebugLog(self, "UnknownDestroyerName=" $ UnknownDestroyerName);
	class'SBMutatorFastWarShell'.static.DebugLog(self, "DestroyMessage=" $ DestroyMessage);
	class'SBMutatorFastWarShell'.static.DebugLog(self, "LaunchSoundName=" $ LaunchSoundName);
	class'SBMutatorFastWarShell'.static.DebugLog(self, "LaunchMessage=" $ LaunchMessage);
	class'SBMutatorFastWarShell'.static.DebugLog(self, "bDestroyedBroadcastMsg=" $ bDestroyedBroadcastMsg);
	class'SBMutatorFastWarShell'.static.DebugLog(self, "bDestroyedBroadcastMsgCritical=" $ bDestroyedBroadcastMsgCritical);
	for (i = 0; i < ArrayCount(ValidWeaponInstigators); i++)
		class'SBMutatorFastWarShell'.static.DebugLog(self, "ValidWeaponInstigators[" $ i $ "]=" $ ValidWeaponInstigators[i]);
	class'SBMutatorFastWarShell'.static.DebugLog(self, "== /INI values ==");
}



simulated function PostBeginPlay() {
local SBMutatorFastWarShell M;

	if (bDebug && Role == ROLE_Authority)
	{
		class'SBMutatorFastWarShell'.static.DebugLog(self, "owner=" $ owner $ ", Instigator=" $ Instigator);
		if ( ! class'SBMutatorFastWarShell'.static.FindMyMutator(self, M))
			warn(self $ "PostBeginPlay DEBUG: my Mutator NOT found");
		else
			if ( ! M.bSBWarShellINIShown) // log only once
			{
				class'SBMutatorFastWarShell'.static.DebugLogProjectileINIValues(self);
				M.bSBWarShellINIShown = true;
			}
	}

	if (bDebug)
		class'SBMutatorFastWarShell'.static.DebugLog(self, "PostBeginPlay: Instigator=" $ Instigator $ ", IsValidWeaponInstigator(Instigator)=" $ IsValidWeaponInstigator(Instigator));

	if (Instigator != None && IsValidWeaponInstigator(Instigator))
	{
		bIsValidWeaponInstigator = true;
		InstigatorHumanName = Instigator.GetHumanName();
		if (bDebug)
			class'SBMutatorFastWarShell'.static.DebugLog(self, "PostBeginPlay: about setting Speed from" @ Speed @ "to" @ ProjectileSpeed);
		Speed = class'SBMutatorFastWarShell'.static.GetSpeed(ProjectileSpeed, class'WarShell'.default.Speed, string(self.name));
		if (bDebug)
			class'SBMutatorFastWarShell'.static.DebugLog(self, "PostBeginPlay: Speed set to" @ Speed);
		MaxSpeed = class'SBMutatorFastWarShell'.static.GetMaxSpeed(Speed, ProjectileMaxSpeed, string(self.name));
		if (Speed > MaxSpeed)
			MaxSpeed = Speed;
		class'SBMutatorFastWarShell'.static.ProjectileLaunched(self, LaunchSoundName, LaunchMessage, Role);
		// super.PostBeginPlay(); *WarShell* only sets timer there
		SmokeRate = 152/Speed;
		SetTimer(SmokeRate, false);
	}
	else
	{
		bIsValidWeaponInstigator = false;
		super.PostBeginPlay();
	}
	if (bDebug)
	{
		//log(self $ "PostBeginPlay finished with SmokeRate=" $ SmokeRate $ ", ProjectileSpeed=" $ ProjectileSpeed  $ ", Speed=" $ Speed);
		class'SBMutatorFastWarShell'.static.DebugLog(self, "PostBeginPlay finished with SmokeRate=" $ SmokeRate $ ", ProjectileSpeed=" $ ProjectileSpeed  $ ", Speed=" $ Speed);
	}

}



simulated function PostNetBeginPlay() {
	if (bDebug)
		class'SBMutatorFastWarShell'.static.DebugLog(self, ".PostNetBeginPlay: Speed=" $ Speed);
	super.PostNetBeginPlay();
}



singular function TakeDamage(int NDamage, Pawn instigatedBy, Vector hitlocation, vector momentum, name damageType) {

	class'SBMutatorFastWarShell'.static.ProjectileTakeDamage(self, ProjectileHealth, bIsValidWeaponInstigator, InstigatorHumanName, NDamage, instigatedBy, UnknownDestroyerName, DestroyMessage, hitlocation, momentum, damageType, bDestroyedBroadcastMsg, bDestroyedBroadcastMsgCritical, bDebug);
}



auto state Flying
{
	function ProcessTouch (Actor Other, Vector HitLocation) {
		if (bApplyFullDamage && Other != Instigator)
			other.TakeDamage(Damage, Instigator, HitLocation, MomentumTransfer * Normal(Velocity), MyDamageType);
		super.ProcessTouch (Other, HitLocation);
	}
}




defaultproperties {
	bApplyFullDamage=true
	bDebug=false
	bDestroyedBroadcastMsg=false
	bDestroyedBroadcastMsgCritical=true
	DestroyMessage="%o destroyed %k's Redeemer"
	LaunchMessage="%k fired a Redeemer"
	LaunchSoundName="SBIncomingSndsV0.NuclearLaunchDetected"
	ProjectileHealth=5
	ProjectileMaxSpeed=10000
	ProjectileSpeed=3000
	UnknownDestroyerName="Someone"
	ValidWeaponInstigators(0)="PlayerPawn"
	ValidWeaponInstigators(1)="Bots"
	ValidWeaponInstigators(2)="Bot"
}
